
from PyQt5 import QtWidgets, QtGui, QtCore
from libnam import control
from libabr import res, app as apps
import importlib

class Run (QtWidgets.QMainWindow):
    ictitle = None
    model = control.read_record("model","/etc/gui")
    model = model.split("x")
    mw = int(model[0]) # width of model
    mh = int(model[1]) # height of model
    username = 'guest'

    '''
    def disableMaximizeButton (self):
        self.actionMax.setVisible(False)

    def disableMinimizeButton (self):
        self.actionMin.setVisible(False)

    def enableMaximizeButton (self):
        self.actionMax.setVisible(True)

    def enableMinimizeButton (self):
        self.actionMin.setVisible(True)
    '''

    def resize (self,w,h):
        self.w = w
        self.h = h
        self.setGeometry(int(self.mw / 2) - int(self.w / 2), int(self.mh / 2) - int(self.h / 2), self.w,self.h)  # Center of screen at all
        self.lbl_title.setGeometry(self.size, 0, w - (self.size * 3), self.size)
        self.btnMin.setGeometry(w-(self.size*3),0, self.size, self.size)
        self.btnMax.setGeometry(w-(self.size*2), 0, self.size, self.size)
        self.btnEscape.setGeometry(w-self.size, 0, self.size, self.size)
        self.titlebar.setFixedSize(w,self.size)

    def setWindowTitle(self,title):
        self.lbl_title.setText(title)

    def setWindowIcon (self,icon):
        self.btnLogo.setIcon(icon)

    def disableMaximizeButton (self):
        self.btnMax.setVisible(False)

    def disableMinimizeButton (self):
        self.btnMin.setVisible(False)

    def enableMaximizeButton(self):
        self.btnMax.setVisible(True)

    def enableMinimizeButton(self):
        self.btnMin.setVisible(True)

    def activate (self):
        # https://stackoverflow.com/questions/3821743/how-to-bring-the-widget-bring-to-front-in-qt #
        self.activateWindow()
        self.raise_()
        apps.switch(self.ID)

    def __init__(self,backend,env,Desktop,mainApp,ID):
        super(Run, self).__init__()

        self.Backend = backend
        self.Env = env
        self.Desktop = Desktop
        self.MainApp = mainApp
        self.ID = ID

        self.username = env.username

        self.setObjectName('app')

        ## Start id process ##
        apps.start (self.ID)

        ## Resize ##

        ## Get all databases ##
        self.default_icon = res.get(control.read_record("app.icon.default-icon", "/etc/gui"))
        self.bgcolor = control.read_record("app.title.bgcolor", "/etc/gui")
        self.fgcolor = control.read_record("app.title.fgcolor", "/etc/gui")
        self.fontsize = control.read_record("fontsize", "/etc/gui")

        self.setStyleSheet('background-color: white;color: black;')

        self.titlebar = QtWidgets.QMainWindow()

        ## Set Colors for TitleBar ##
        if not self.bgcolor == None and not self.fgcolor == None:
            self.titlebar.setStyleSheet("background-color: " + self.bgcolor + ";color: " + self.fgcolor + ";")
        elif not self.bgcolor == None:
            if self.bgcolor == "#FFFFFF" or self.bgcolor == "white":
                self.titlebar.setStyleSheet("background-color: " + self.bgcolor + ";color: black;")
            else:
                self.title.setStyleSheet("background-color: " + self.bgcolor + ";color: white;")
        elif not self.fgcolor == None:
            self.titlebar.setStyleSheet("background-color: #00BFFF;color: " + self.fgcolor)
        else:
            self.titlebar.setStyleSheet("background-color: #00BFFF;color: white;")

        ## Get icons ##
        self.locale = control.read_record("locale","/etc/gui")
        self.escapei = res.get(control.read_record("app.icon.escape", "/etc/gui"))
        self.size = control.read_record("app.title.size",'/etc/gui')
        
        if self.size==None:
            self.size = int(self.size)
        else:
            self.size = int(self.size)

        ## layout of title ##

        ## Logo ##

        ## Minimize Button ##
        self.btnLogo = QtWidgets.QPushButton()
        self.btnLogo.setIcon(QtGui.QIcon(res.get('@logo/app')))
        self.btnLogo.setGeometry(0, 0, self.size, self.size)
        self.btnLogo.setIconSize(QtCore.QSize(self.size,self.size))
        self.btnLogo.setStyleSheet('border-radius: '+str(self.size/2)+';')
        self.titlebar.layout().addWidget(self.btnLogo)

        ## Text of Title ##
        self.lbl_title = QtWidgets.QLabel()
        self.lbl_title.setText("My Window")
        self.lbl_title.setStyleSheet('color: white;')
        self.lbl_title.setGeometry(self.size,0,self.width()-(self.size*3),self.size)
        self.titlebar.layout().addWidget(self.lbl_title)

        ## Minimize Button ##
        self.btnMin = QtWidgets.QPushButton()
        self.btnMin.setIcon(QtGui.QIcon(res.get('@logo/min')))
        self.btnMin.setGeometry(self.width()-400,0,self.size,self.size)
        self.btnMin.setStyleSheet('border-radius: '+str(self.size/2)+'% '+str(self.size/2)+'%;')
        self.btnMin.clicked.connect(self.actMin)
        self.titlebar.layout().addWidget(self.btnMin)

        ## Minimize Button ##
        self.btnMax= QtWidgets.QPushButton()
        self.btnMax.setIcon(QtGui.QIcon(res.get('@logo/max')))
        self.btnMax.setGeometry(self.width()-360, 0, self.size, self.size)
        self.btnMax.setStyleSheet('border-radius: '+str(self.size/2)+'% '+str(self.size/2)+'%;')
        self.btnMax.clicked.connect(self.actMax)
        self.titlebar.layout().addWidget(self.btnMax)

        ## Minimize Button ##
        self.btnEscape = QtWidgets.QPushButton()
        self.btnEscape.setIcon(QtGui.QIcon(self.escapei))
        self.btnEscape.setGeometry(self.width()-320, 0, self.size, self.size)
        self.btnEscape.setStyleSheet('border-radius: '+str(self.size/2)+'% '+str(self.size/2)+'%;')
        self.btnEscape.clicked.connect(self.actClose)
        self.titlebar.layout().addWidget(self.btnEscape)

        self.mainApp = self.MainApp.MainApp([self.Backend,self.Env,self.Desktop,self])
        self.mainApp.setGeometry (0,self.size,self.w,self.height()-self.size)
        self.layout().addWidget(self.mainApp)  # Run widget
        self.layout().addWidget(self.titlebar)


    def actClose(self):
        apps.end(self.ID)
        self.close()

    def actMin(self):
        self.hide()

    maxf = False
    h = 0
    w = 0

    def actMax (self):
        if self.maxf==False:
            self.w = self.width()
            self.h = self.height()

            # Activate Window and switch to this ID process #
            self.activateWindow()
            self.raise_()
            apps.switch (self.ID)

            ## Set taskbar location ##
            if not self.username == "guest":
                self.taskbar_location = control.read_record("desktop.taskbar.location", "/etc/users/" + self.username)
            else:
                self.taskbar_location = None
            if self.taskbar_location == None:
                self.taskbar_location = control.read_record("desktop.taskbar.location", "/etc/gui")
                if self.taskbar_location == None:
                    self.taskbar_location = "left"

            ## Set taskbar size
            if not self.username == "guest":
                self.taskbar_size = control.read_record("desktop.taskbar.size", "/etc/users/" + self.username)
            else:
                self.taskbar_size = None

            if self.taskbar_size == None:
                self.taskbar_size = control.read_record("desktop.taskbar.size", "/etc/gui")
                if self.taskbar_size == None:
                    self.taskbar_size = 50 + 15
                else:
                    self.taskbar_size = int(self.taskbar_size) + 15

            if self.taskbar_location == "left" or self.taskbar_location=="right":
                self.setGeometry(0, 0, self.mw - self.taskbar_size, self.mh)

                self.btnMin.setGeometry(self.mw- self.taskbar_size - (self.size * 3), 0, self.size, self.size)
                self.btnMax.setGeometry(self.mw- self.taskbar_size - (self.size * 2), 0, self.size, self.size)
                self.btnEscape.setGeometry(self.mw - self.taskbar_size - self.size, 0, self.size, self.size)
                self.titlebar.setFixedSize(self.mw - self.taskbar_size, self.size)
                self.mainApp.setGeometry(0, self.size, self.mw - self.taskbar_size, self.mh)

            elif self.taskbar_location == "bottom" or self.taskbar_location=="top":
                self.setGeometry(0, 0, self.mw, self.mh - self.taskbar_size)
                self.lbl_title.setGeometry(self.size, 0, self.mw - (self.size * 3), self.size)

                self.btnMin.setGeometry(self.mw - (self.size * 3), 0, self.size, self.size)
                self.btnMax.setGeometry(self.mw - (self.size * 2), 0, self.size, self.size)
                self.btnEscape.setGeometry(self.mw - self.size, 0, self.size, self.size)
                self.titlebar.setFixedSize(self.mw, self.size)
                self.mainApp.setGeometry(0, self.size, self.mw, self.mh - self.taskbar_size)

            else:
                self.setGeometry(0, 0, self.mw, self.mh - self.taskbar_size)

                self.btnMin.setGeometry(self.mw - (self.size * 3), 0, self.size, self.size)
                self.btnMax.setGeometry(self.mw - (self.size * 2), 0, self.size, self.size)
                self.btnEscape.setGeometry(self.mw - self.size, 0, self.size, self.size)
                self.titlebar.setFixedSize(self.mw, self.size)
                self.mainApp.setGeometry(0, self.size, self.mw, self.mh - self.taskbar_size)

            self.maxf = True
            self.btnMax.setIcon(QtGui.QIcon(res.get(control.read_record("app.icon.max-min", "/etc/gui"))))
        else:
            self.maxf = False
            self.setGeometry(int(self.mw/2)-int(self.w/2), int(self.mh/2)-int(self.h/2),self.w,self.h) # Center of screen at all
            self.btnMax.setIcon(QtGui.QIcon(res.get(control.read_record("app.icon.max", "/etc/gui"))))
            self.btnMin.setGeometry(self.w - (self.size * 3), 0, self.size, self.size)
            self.btnMax.setGeometry(self.w - (self.size * 2), 0, self.size, self.size)
            self.btnEscape.setGeometry(self.w - self.size, 0, self.size, self.size)
            self.titlebar.setFixedSize(self.w, self.size)
            self.mainApp.setGeometry(0,self.size,self.w,self.h-self.size)

    ## Customize the window  (stackoverflow.com) ##
    def mousePressEvent(self, event):
        if event.button() == QtCore.Qt.LeftButton:
            self.activateWindow()
            self.raise_()
            apps.switch(self.ID)
            self.__press_pos = event.pos()

    def mouseReleaseEvent(self, event):
        if event.button() == QtCore.Qt.LeftButton:
            self.__press_pos = None

    def mouseMoveEvent(self, event):
        if self.__press_pos:
            self.move(self.pos() + (event.pos() - self.__press_pos))