from PyQt5.QtWidgets import *
from PyQt5 import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5 import uic
from libnam import files, control, process
import importlib, hashlib
from importlib import reload
from libabr import core, file, res

from PyQt5 import QtCore, QtGui, QtWidgets

class MainApp (QMainWindow):
    fontsize = control.read_record("fontsize", "/etc/gui") # Get font size
    model = control.read_record("model", "/etc/gui") # Get screen model
    locale = control.read_record("locale","/etc/gui") # Get locale
    model = model.split("x")
    mw = int(model[0])  # width of model
    mh = int(model[1])  # height of model

    def suspend_act(self):
        import suspendpage
        importlib.reload(suspendpage) ##https://www.geeksforgeeks.org/reloading-modules-python/
        self.setCentralWidget(suspendpage.MainApp(self))

    def escape_act(self):
        core.shutdown()
        exit(0)

    def restart_act(self):
        core.reboot()
        exit(0)

    def enter(self):
        self.close()
        import enter
        self.Backend.setCentralWidget(enter.MainApp(self.Backend))

    def desktop(self):
        self.close()
        import desktop
        self.Backend.setCentralWidget(desktop.MainApp (self.Backend))

    def check_user(self):
        self.username = self.lineEdit.text().lower()
        self.enable_gui = control.read_record("enable_gui", "/etc/guest")

        ## If user is guest
        if self.username == "guest" and self.enable_gui == "Yes":
            self.lineEdit.setEnabled(False)
            files.write("/tmp/username.tmp", self.username)
            files.create("/tmp/su.tmp")
            control.write_record("username", self.username, "/tmp/su.tmp")
            control.write_record("code", "", "/tmp/su.tmp")
            QTimer.singleShot(0,self.desktop)

        ## If user is x
        elif files.isfile("/etc/users/" + self.username):
            self.hashname = hashlib.sha3_256(
                str(self.username).encode()).hexdigest()

            self.user = control.read_record("username", "/etc/users/" + self.username)

            if self.user == self.hashname:
                files.write("/tmp/username.tmp", self.username)
                QTimer.singleShot(1000,self.enter)
            else:
                self.lblDown.setText(res.get("@string/lblUserNotFound").replace("{0}", self.username))
                QTimer.singleShot(1000, self.clear)
        else:
            self.lblDown.setText(res.get("@string/lblUserNotFound").replace("{0}", self.username))
            QTimer.singleShot(1000, self.clear)

    ## Clear wrongs
    def clear(self):
        self.lblDown.clear()
        self.lineEdit.clear()

    def __init__(self,Backend):
        super().__init__()
        self.Backend = Backend
        ## Load ui ##
        #uic.loadUi(res.get('@widget/default'),self)
        self.setObjectName('default')

        self.setFixedSize(self.mw,self.mh)

        ## MenuBar ##
        self.menubar = QMenuBar()
        self.setMenuBar(self.menubar)

        ## Frameless the window ##
        self.setWindowFlags(QtCore.Qt.FramelessWindowHint)

        ## Background image button ##
        self.btnBackgroundImage = QtWidgets.QPushButton()
        self.setCentralWidget(self.btnBackgroundImage)

        ## Finds ##
        #self.btnAccountImage = self.findChild(QPushButton,'btnAccountImage')
        #self.lblTop = self.findChild(QLabel, 'lblTop')
        #self.lblDown = self.findChild(QLabel,'lblDown')
        self.menubar.setStyleSheet("background: none;")
        self.menuPower = self.menubar.addMenu('')
        self.actionEscape = self.menuPower.addAction('')
        self.actionRestart = self.menuPower.addAction('')
        self.actionSuspend = self.menuPower.addAction('')

        ## Font Size configure ##
        if self.fontsize == None:
            self.fontsize = 12
        else:
            self.fontsize = int(self.fontsize)

        ## Set layout direction

        if res.get("@string/menu_direction") == "rtl":
            self.menubar.setLayoutDirection(QtCore.Qt.RightToLeft)
        else:
            self.menubar.setLayoutDirection(QtCore.Qt.LeftToRight)

        ## Set locales all ##
        #self.lblTop.setText(res.get("@string/lblTop"))
        self.menuPower.setTitle(res.get("@string/menu_power"))
        self.actionRestart.setText(res.get("@string/action_restart"))
        self.actionEscape.setText(res.get("@string/action_escape"))
        self.actionSuspend.setText(res.get("@string/action_suspend"))

        ## Set fonts all ##
        #self.lblDown.setFont (QFont(res.get("@string/font"),self.fontsize))
        #self.lblTop.setFont(QFont(res.get("@string/font"), self.fontsize))
        self.menubar.setFont(QFont(res.get("@string/font"), self.fontsize))
        self.menuPower.setFont(QFont(res.get("@string/font"), self.fontsize))
        self.actionSuspend.setFont(QFont(res.get("@string/font"), self.fontsize))
        self.actionRestart.setFont(QFont(res.get("@string/font"), self.fontsize))
        self.actionEscape.setFont(QFont(res.get("@string/font"), self.fontsize))

        ## Set Actions ##
        self.actionEscape.triggered.connect (self.escape_act)
        self.actionRestart.triggered.connect(self.restart_act)
        self.actionSuspend.triggered.connect(self.suspend_act)

        ## PyCloud data
        self.cs = files.readall("/proc/info/cs")
        self.cd = files.readall("/proc/info/cd")
        self.ver = files.readall("/proc/info/ver")
        self.setWindowTitle(self.cs + " " + self.ver + " (" + self.cd + ")")

        ## Default bgcolor check ##
        bgcolor = control.read_record("login.bgcolor", "/etc/gui")
        background = res.get(control.read_record("login.background","/etc/gui"))

        if not bgcolor == None:
            self.btnBackgroundImage.setStyleSheet("border: none;background-color: " + bgcolor + ";")
        elif not background == None:
            self.btnBackgroundImage.setStyleSheet("border: none;background-image: url(" + background + ");")
        else:
            self.btnBackgroundImage.setStyleSheet("border: none;background-color: purple;")

        ## Float login widget ##
        self.loginw = QWidget()
        uic.loadUi(res.get('@widget/enter'),self.loginw)
        self.loginw.setGeometry(int(self.mw / 2) - int(self.loginw.width() / 2),int(self.mh / 15),self.loginw.width(),self.loginw.height())
        self.layout().addWidget(self.loginw)

        ## Finds ##
        self.btnAccountImage = self.findChild(QtWidgets.QPushButton,'btnAccountImage')
        self.btnAccountImage.setStyleSheet ('background-image: url('+res.get(control.read_record("login.userlogo","/etc/gui"))+');border-radius: 128% 128%;border-style: solid;border-color: gray;border-width: 1%')

        self.lblTop = self.findChild(QtWidgets.QLabel,'lblTop')
        self.lblTop.setText (res.get("@string/lblUsername"))
        self.lblTop.setFont(QFont(res.get("@string/font"), self.fontsize))
        self.lblDown = self.findChild(QtWidgets.QLabel,'lblDown')
        self.lblDown.setFont(QFont(res.get("@string/font"), self.fontsize))
        self.lineEdit = self.findChild(QtWidgets.QLineEdit,'lineEdit')
        self.lineEdit.setFont(QFont(res.get("@string/font"), self.fontsize))
        self.lineEdit.returnPressed.connect (self.check_user)

        ## Show default ##
        self.showFullScreen()